﻿using System;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;


namespace  GPStoGoogleMapDesktop
{
    public partial class TestForm : Form
    { 
        readonly static string _bingCacheDirectory = @"C:\bing cache\cache\";
        readonly FileBitmapTileCache _cache = new FileBitmapTileCache(_bingCacheDirectory);

        public TestForm()
        {
            InitializeComponent();
        }

        private void TestForm_Load(object sender, EventArgs e)
        {
           
            winformsMap1.ThreadingMode = MapThreadingMode.SingleThreaded;
            
            winformsMap1.MapUnit = GeographyUnit.Meter;
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 198, 255, 255));
                        
            BingMapsLayer bingLayer = new BingMapsLayer("Ai7suMEtJnFbj-QTaVH6TwJ29nYOjN6udUzR42VNna_QTMURVOYZulMni6U48qJT", BingMapsMapType.Aerial);     
            bingLayer.LogoMode = BingMapsLogoMode.NoLogo;
            bingLayer.CacheDirectory = _bingCacheDirectory;
            bingLayer.DrawingExceptionMode = DrawingExceptionMode.ThrowException;
                        
            MyLayerOverlay bingLayerOverlay = new MyLayerOverlay();
            bingLayerOverlay.Layers.Add(bingLayer);
            bingLayerOverlay.DrawingExceptionMode = DrawingExceptionMode.ThrowException;
            
            winformsMap1.Overlays.Add(bingLayerOverlay);

            //winformsMap1.ZoomLevelSet = new GoogleMapZoomLevelSet();            
            winformsMap1.ZoomLevelSet = new BingMapsZoomLevelSet();

            InMemoryFeatureLayer pointLayer = new InMemoryFeatureLayer();
            pointLayer.Open();
            pointLayer.Columns.Add(new FeatureSourceColumn("Text"));
            pointLayer.Close();

            //Sets the projection parameters to go from Geodetic (EPSG 4326) or decimal degrees to Google Map projection (Spherical Mercator).
            Proj4Projection proj4 = new Proj4Projection();
            proj4.InternalProjectionParametersString = Proj4Projection.GetEpsgParametersString(4326);
            proj4.ExternalProjectionParametersString = Proj4Projection.GetGoogleMapParametersString();
            //Applies the projection to the InMemoryFeatureLayer so that the point in decimal degrees (Longitude/Latitude) can be 
            //match the projection of Google Map.
            pointLayer.FeatureSource.Projection = proj4;

            //Values in Longitude and Latitude.
            double Longitude = -93.789018;
            double Latitude = 41.601151;

            //Creates the feature made of a PointShape with the Longitude and Latitude values.
            Feature GPSFeature = new Feature(new PointShape(Longitude, Latitude));

            //Format the Longitude and Latitude into a nice string as Degrees Minutes and Seconds
            string LongLat = DecimalDegreesHelper.GetDegreesMinutesSecondsStringFromDecimalDegreePoint(GPSFeature);

            //Sets the InMemoryFeatureLayer to have it displayed with Square symbol and with text.
            GPSFeature.ColumnValues.Add("Text", "Boston's    " + LongLat);
            pointLayer.InternalFeatures.Add(GPSFeature);

            pointLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Square,
                                                                    GeoColor.StandardColors.Red, GeoColor.StandardColors.Black, 2, 12);
            pointLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.CreateSimpleTextStyle("Text", "Arial", 12, DrawingFontStyles.Bold,
                                                                    GeoColor.StandardColors.Black, GeoColor.StandardColors.White, 3, -10, 10);
            pointLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay pointOverlay = new LayerOverlay();
            pointOverlay.Layers.Add("PointLayer", pointLayer);            


            ShapeFileFeatureLayer polkShapeLayer = new ShapeFileFeatureLayer(@"C:\bing cache\IA\Polk\IA153ctycu.shp");
            polkShapeLayer.ZoomLevelSet = new BingMapsZoomLevelSet();
            polkShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Circle, GeoColor.StandardColors.Red, GeoColor.StandardColors.Black, 9);
            polkShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.CreateMaskTextStyle("NAME", new GeoFont("Arial", 10, DrawingFontStyles.Bold),
            new GeoSolidBrush(GeoColor.StandardColors.Black), new AreaStyle(new GeoSolidBrush(GeoColor.StandardColors.LightGoldenrodYellow)), 13, 0);
            polkShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(50, GeoColor.SimpleColors.Yellow), GeoColor.StandardColors.Yellow, 1, LineDashStyle.Solid);
            polkShapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            //Sets the projection to the shapefile layer.
            polkShapeLayer.FeatureSource.Projection = proj4;

            ShapeFileFeatureLayer dallasShapeLayer = new ShapeFileFeatureLayer(@"C:\bing cache\IA\Dallas\IA049ctycu.shp");
            dallasShapeLayer.ZoomLevelSet = new BingMapsZoomLevelSet();
            dallasShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Circle, GeoColor.StandardColors.Red, GeoColor.StandardColors.Black, 9);
            dallasShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.CreateMaskTextStyle("NAME", new GeoFont("Arial", 10, DrawingFontStyles.Bold),
            new GeoSolidBrush(GeoColor.StandardColors.Black), new AreaStyle(new GeoSolidBrush(GeoColor.StandardColors.LightGoldenrodYellow)), 13, 0);
            dallasShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(50, GeoColor.SimpleColors.Yellow), GeoColor.StandardColors.Yellow, 1, LineDashStyle.Solid);
            dallasShapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            //Sets the projection to the shapefile layer.
            dallasShapeLayer.FeatureSource.Projection = proj4;

            ShapeFileFeatureLayer siouxShapeLayer = new ShapeFileFeatureLayer(@"C:\bing cache\IA\Sioux\IA167ctycu.shp");
            siouxShapeLayer.ZoomLevelSet = new BingMapsZoomLevelSet();
            siouxShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Circle, GeoColor.StandardColors.Red, GeoColor.StandardColors.Black, 9);
            siouxShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.CreateMaskTextStyle("NAME", new GeoFont("Arial", 10, DrawingFontStyles.Bold),
            new GeoSolidBrush(GeoColor.StandardColors.Black), new AreaStyle(new GeoSolidBrush(GeoColor.StandardColors.LightGoldenrodYellow)), 13, 0);
            siouxShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(50, GeoColor.SimpleColors.Yellow), GeoColor.StandardColors.Yellow, 1, LineDashStyle.Solid);
            siouxShapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            //Sets the projection to the shapefile layer.
            siouxShapeLayer.FeatureSource.Projection = proj4;

            LayerOverlay staticOverlay = new LayerOverlay();
            staticOverlay.Layers.Add("polkShapeLayer", polkShapeLayer);
            staticOverlay.Layers.Add("dallasShapeLayer", dallasShapeLayer);
            staticOverlay.Layers.Add("siouxShapeLayer", siouxShapeLayer);
            
            winformsMap1.Overlays.Add(staticOverlay);
            winformsMap1.Overlays.Add("PointOverlay", pointOverlay);

            //Add a field
            string wkt = "POLYGON ((-96.12738699548338 43.142020002252984,-96.12734101269064 43.140485004911156,-96.1273250121209 43.13940499772093,-96.12738000292126 43.1391209998834,-96.1273690099484 43.13909099106735,-96.12734799484141 43.13907800302802,-96.12708100843477 43.13905600754453,-96.12320100879145 43.139091009956566,-96.11819499337054 43.13915499539665,-96.11781301052363 43.139154998857016,-96.11751400491413 43.13914299913993,-96.11746900842702 43.139157008518254,-96.1174560022105 43.1391910022309,-96.11752299249069 43.13945699503034,-96.11753398930966 43.14031700455982,-96.11759700156537 43.14237699280417,-96.11762101254398 43.142591992023284,-96.11771500320117 43.1426540083656,-96.11786698696591 43.142692997497306,-96.1179780072292 43.14269900168033,-96.11898800235541 43.1427119984208,-96.1204469954831 43.142701000390275,-96.12147099570674 43.142683004871046,-96.12255101042884 43.14267999888865,-96.12364999619784 43.142658994358975,-96.1273550059777 43.14261500269588,-96.12740801104927 43.14260000142729,-96.12742201291472 43.142569007537006,-96.12738998854448 43.142323998227354,-96.12738801242217 43.14207699854265,-96.12738699548338 43.142020002252984))";
            LayerOverlay fieldsOverlay = new LayerOverlay();
            InMemoryFeatureLayer fieldLayer = new InMemoryFeatureLayer();
            fieldLayer.ZoomLevelSet = new BingMapsZoomLevelSet();
            fieldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(50, GeoColor.SimpleColors.Red), GeoColor.StandardColors.Red, 1, LineDashStyle.Solid);
            fieldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            fieldLayer.FeatureSource.Projection = proj4;
           
            Feature field = new PolygonShape(wkt).GetFeature();
            field.ColumnValues.Add("FieldName", "TestField");

            fieldLayer.Open();
            List<string> columns = new List<string>(field.ColumnValues.Keys);

            foreach(string columnName in columns)
                fieldLayer.Columns.Add(new FeatureSourceColumn(columnName));

            fieldLayer.InternalFeatures.Add(field);
                       
            fieldLayer.Close();

            fieldsOverlay.Layers.Add(fieldLayer);
            winformsMap1.Overlays.Add(fieldsOverlay);

            siouxShapeLayer.Open();
            winformsMap1.CurrentExtent = siouxShapeLayer.GetBoundingBox();
            siouxShapeLayer.Close();

            winformsMap1.ZoomLevelSnapping = ZoomLevelSnappingMode.SnapUp;                                 

            winformsMap1.Refresh();
        }

      
        private void winformsMap1_MouseMove(object sender, MouseEventArgs e)
        {
            //Displays the X and Y in screen coordinates.
            statusStrip1.Items["toolStripStatusLabelScreen"].Text = "X:" + e.X + " Y:" + e.Y;
        
            //Gets the PointShape in world coordinates from screen coordinates.
            PointShape pointShape = ExtentHelper.ToWorldCoordinate(winformsMap1.CurrentExtent, new ScreenPointF(e.X, e.Y), winformsMap1.Width, winformsMap1.Height);

            //Displays world coordinates.
            statusStrip1.Items["toolStripStatusLabelWorld"].Text = "(world) X:" + Math.Round(pointShape.X, 4) + " Y:" + Math.Round(pointShape.Y, 4);
                     
        }
        
        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnClearCache_Click(object sender, EventArgs e)
        {
            System.IO.Directory.Delete(_bingCacheDirectory, true);            
            _cache.ClearCache();            
        }
    }
}
